@@ -4,7 +4,8 @@
 
 ;; Author: Milan Glacier <dev@milanglacier.com>
 ;; Maintainer: Milan Glacier <dev@milanglacier.com>
-;; Version: 0.6.0
+;; Package-Version: 0.7.1
+;; Package-Revision: v0.7.1-0-g7b34bf0f0334
 ;; URL: https://github.com/milanglacier/minuet-ai.el
 ;; Package-Requires: ((emacs "29") (plz "0.9") (dash "2.19.1"))
 
@@ -61,10 +62,13 @@
   "Debounce delay in seconds for auto-suggestions."
   :type 'number)
 
-(defcustom minuet-auto-suggestion-block-functions '(minuet-evil-not-insert-state-p)
+(define-obsolete-variable-alias 'minuet-auto-suggestion-block-functions
+  'minuet-auto-suggestion-block-predicates "0.7.1")
+
+(defcustom minuet-auto-suggestion-block-predicates '(minuet-evil-not-insert-state-p)
   "List of functions to determine whether auto-suggestions should be blocked.
 
-Each function should return non-nil if auto-suggestions should be
+Each functions should return non-nil if auto-suggestions should be
 blocked.  If any function in this list returns non-nil,
 auto-suggestions will not be shown."
   :type '(repeat function))
@@ -81,6 +85,8 @@ auto-suggestions will not be shown."
 (defvar-local minuet--current-overlay nil
   "Overlay used for displaying the current suggestion.")
 
+(defvar-local minuet--last-synced-point nil
+  "Last point where typed text was synced with the suggestion overlay.")
 
 (defvar-local minuet--last-point nil
   "Last known cursor position for suggestion overlay.")
@@ -489,10 +495,38 @@ Also cancel any pending requests unless NO-CANCEL is t."
   (and minuet--last-point
        (not (eq minuet--last-point (point)))))
 
+(defun minuet--sync-suggestion-with-typed-text ()
+  "Update the suggestion when the typed text matches its prefix.
+Return non-nil when the completion was preserved or updated."
+  (if-let* ((suggestions minuet--current-suggestions)
+            (has-overlay minuet--current-overlay)
+            (last-point minuet--last-point)
+            (cursor-moved-forward (> (point) last-point))
+            (index (or minuet--current-suggestion-index 0))
+            (current-suggestion (nth index suggestions))
+            (typed (buffer-substring-no-properties last-point (point)))
+            (current-suggestion-matches-typed
+             (and (string-prefix-p typed current-suggestion)
+                  (length> current-suggestion (length typed))))
+            (matched-suggestions
+             (cl-loop for suggestion in suggestions
+                      for i from 0
+                      when (string-prefix-p typed suggestion)
+                      collect (cons i (substring suggestion (length typed)))))
+            (new-index (or (cl-position index matched-suggestions :key #'car) 0))
+            (new-suggestions (mapcar #'cdr matched-suggestions)))
+      (progn
+        (minuet--cleanup-suggestion)
+        (minuet--display-suggestion new-suggestions new-index)
+        (setq minuet--last-synced-point (point))
+        t)
+    (setq minuet--last-synced-point nil)))
+
 (defun minuet--on-cursor-moved ()
   "Minuet event on cursor moved."
   (when (minuet--cursor-moved-p)
-    (minuet--cleanup-suggestion)))
+    (unless (minuet--sync-suggestion-with-typed-text)
+      (minuet--cleanup-suggestion))))
 
 (defun minuet--display-suggestion (suggestions &optional index)
   "Display suggestion from SUGGESTIONS at INDEX using an overlay at point."
@@ -882,9 +916,21 @@ many lines.  Without a prefix argument, accept only the first line."
                             minuet--current-suggestions))
            (lines (split-string suggestion "\n"))
            (n (or n 1))
-           (selected-lines (seq-take lines n)))
+           (selected-lines (seq-take lines n))
+           (remaining-lines (seq-drop lines n))
+           (remaining-suggestion (when remaining-lines
+                                   (string-join remaining-lines "\n"))))
       (minuet--cleanup-suggestion)
-      (insert (string-join selected-lines "\n")))))
+      (insert (string-join selected-lines "\n"))
+      (when remaining-suggestion
+        (insert "\n") ;; There is a remaining suggestion, so move to the next line.
+        ;; NOTE: We do not need to worry about Minuet
+        ;; automatically triggering the next suggestion upon
+        ;; continuous acceptance.  Minuet only attempts
+        ;; auto-suggestion when the last command was not a Minuet
+        ;; command. Since we have just accepted partial lines, the
+        ;; last command is indeed a Minuet command.
+        (minuet--display-suggestion (list remaining-suggestion) 0)))))
 
 ;;;###autoload
 (defun minuet-complete-with-minibuffer ()
@@ -1307,7 +1353,8 @@ to be called when completion items arrive."
 (defun minuet--maybe-show-suggestion ()
   "Show suggestion with debouncing and throttling."
   (when (and (minuet--is-not-on-throttle)
-             (not (minuet--is-minuet-command)))
+             (not (minuet--is-minuet-command))
+             (not (eq (point) minuet--last-synced-point)))
     (when minuet--debounce-timer
       (cancel-timer minuet--debounce-timer))
     (setq minuet--debounce-timer
@@ -1318,7 +1365,8 @@ to be called when completion items arrive."
                (when (and (eq buffer (current-buffer))
                           (or (null minuet--auto-last-point)
                               (not (eq minuet--auto-last-point (point))))
-                          (not (run-hook-with-args-until-success 'minuet-auto-suggestion-block-functions)))
+                          (not (run-hook-with-args-until-success
+                                'minuet-auto-suggestion-block-predicates)))
                  (setq minuet--last-auto-suggestion-time (current-time)
                        minuet--auto-last-point (point))
                  (minuet-show-suggestion))))))))
@@ -1391,14 +1439,9 @@ settings, as an alternative to manual configuration via `setq' and
 `openai-fim-compatible' providers, users will be prompted to specify
 their endpoint and API key."
   (interactive)
-  (let* ((providers '(("OpenAI" . openai)
-                      ("Claude" . claude)
-                      ("Codestral" . codestral)
-                      ("OpenAI Compatible" . openai-compatible)
-                      ("OpenAI FIM Compatible" . openai-fim-compatible)
-                      ("Gemini" . gemini)))
+  (let* ((providers (mapcar #'cddr (cdr (get 'minuet-provider 'custom-type))))
          (provider-name (completing-read "Select provider: " providers nil t))
-         (provider (alist-get provider-name providers nil nil #'equal))
+         (provider (car (alist-get provider-name providers nil nil #'equal)))
          (options-sym (intern (format "minuet-%s-options" provider)))
          (options (symbol-value options-sym))
          (current-model (plist-get options :model))
